#include "listclass.h"

Mailbox::Mailbox()
{
	this->first = new box;
	this->last = this->first;
	this->total = 0;
	InitializeCriticalSection(&mbcs);
}

Mailbox::~Mailbox(void)
{
	EnterCriticalSection(&mbcs);
	box *rem = this->first;
	box *forwardpos;
	while(rem != 0)
	{
		forwardpos = rem->next;
		delete rem;
		rem = forwardpos;
	}
	LeaveCriticalSection(&mbcs);
	DeleteCriticalSection(&mbcs);
}

BOOL Mailbox::Add(char *nick)
{
	EnterCriticalSection(&mbcs);

	box *newnode = this->last;
	this->last->next = new box;
	this->last = this->last->next;

	strcpy(this->last->nick, nick);

	this->last->prev = newnode;
	
	this->total++;
	LeaveCriticalSection(&mbcs);
	return true;
}

BOOL Mailbox::Find(char *nick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			LeaveCriticalSection(&mbcs);
			return true;
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return false;
}

BOOL Mailbox::AppendMsg(std::string msg, char *nick, char *fromnick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			pos->buffer << "\r\nMessage From " << fromnick << ": " << msg;
			LeaveCriticalSection(&mbcs);
			return true;
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return false;
}

std::string Mailbox::GetMessages(char *nick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			LeaveCriticalSection(&mbcs);
			return pos->buffer.str();
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return "No messages found";
}

BOOL Mailbox::DelMessages(char *nick)
{
	EnterCriticalSection(&mbcs);
	box *pos = this->first;
	box *forward;
	while(pos != 0)
	{
		forward = pos->next;
		if(strcmp(pos->nick, nick) == 0)
		{
			pos->buffer.str("");
			pos->buffer << "\r\nMessages: ";
			LeaveCriticalSection(&mbcs);
			return true;
		}
		pos = forward;
	}
	LeaveCriticalSection(&mbcs);
	return false;
}