

#include <string>
#include <sstream>
#include "listclass.h"
#include "HeXHubSDK.h"

long trigId, hlpId, msgchktrigId, msgchkhlpId, msgdltrigId, msgdlhlpId;
Mailbox *Msgbox = new Mailbox;
CRITICAL_SECTION cs;

void HEXHUBAPI onMessageCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
void HEXHUBAPI onChkCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);
void HEXHUBAPI onDelCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM);

const char * HEXHUBAPI InitPlugin(lpfnGetFunction HexGetFunction) {
	//HexHubFunctions functions;
	HexHub Hex;
	
	if (Hex.Initialize(HexGetFunction) == false) return 0;

	if ((msgchktrigId = HexHub::Core::RegisterCommand("checkmsgs", onChkCommand)) == -1) return 0;
	if ((msgdltrigId = HexHub::Core::RegisterCommand("delmsgs", onDelCommand)) == -1) return 0;
	if ((trigId = HexHub::Core::RegisterCommand("message", onMessageCommand)) == -1) return 0;
	
	if ((hlpId = HexHub::Core::RegisterHelpString("EN", "!message <username> <body>\t\t\t-\tSend A Message to a user that is not online", 1)) == -1) {
		HexHub::Core::UnregisterCommand(trigId);
		return 0;
	}

	if ((msgchkhlpId = HexHub::Core::RegisterHelpString("EN", "!checkmsgs\t\t\t-\tCheck your messages", 1)) == -1) {
		HexHub::Core::UnregisterCommand(msgchktrigId);
		return 0;
	}

	if ((msgdlhlpId = HexHub::Core::RegisterHelpString("EN", "!delmsgs\t\t\t-\tDelete your messages", 1)) == -1) {
		HexHub::Core::UnregisterCommand(msgchktrigId);
		return 0;
	}
	InitializeCriticalSection(&cs);
	return "Messenger Plugin 0.1 by The Architect";
}

void HEXHUBAPI onNicklistChange(DWORD dwUserId,BOOL addedToList)
{
	int AccId = 0;
	if((AccId = HexHub::Users::Registration::GetAccountId(dwUserId)) > 0 && addedToList == true)
	{
		char Nck[40];
		EnterCriticalSection(&cs);
		if(HexHub::Users::ClientNickFromId(dwUserId, Nck))
		{
			if(!Msgbox->Find(Nck))
			{
				Msgbox->Add(Nck);
				std::stringstream Sendstr;
				Sendstr << "<Hermes> A mailbox has been added for you. You can check your messages with !checkmsgs :)|";
				HexHub::Data::SendTo(dwUserId, Sendstr.str().c_str(), Sendstr.str().length());
			}
			else
			{
				std::stringstream Sendstr;
				Sendstr << "<Hermes> " << Msgbox->GetMessages(Nck) << "|";
				HexHub::Data::SendTo(dwUserId, Sendstr.str().c_str(), Sendstr.str().length());
			}
		}
		LeaveCriticalSection(&cs);
	}
}

void HEXHUBAPI onMessageCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char SenderNick[41];
	char *body = 0;
	if(HexHub::Users::ClientNickFromId(userId, SenderNick))
	{
		char Parameters[8192];
		int i = 0;
		memset(Parameters, 0, 8192);
		for (i=0; (i < 8191) && (commandParameters[i] != '|') && (commandParameters[i] != 0); i++) Parameters[i] = commandParameters[i];
		for (i=0; Parameters[i] != 0; i++) {
			if (Parameters[i] == ' ') {
				Parameters[i] = 0;
				body = (&Parameters[i]) + 1;
				break;
			}
		}
		std::stringstream SendValid;
		if(Parameters[0] != 0)
		{
			if(body != 0)
			{
				SendValid << "You have chosen to message " << Parameters << " with the body: " << body << "|";
				HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
				SendValid.str("");
				EnterCriticalSection(&cs);
				if(Msgbox->Find(Parameters))
				{
					SendValid << "Send to " << Parameters << " was successful!|";
					HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
					Msgbox->AppendMsg(body, Parameters, SenderNick);
					int id = 0;
					if((id = HexHub::Users::IdFromNick(Parameters)) > -1)
					{
						std::string snd = "<Hermes> You have a new message. Please type !checkmsgs to check your messages!|";
						HexHub::Data::SendTo(id, snd.c_str(), snd.length());
					}
				}
				else
				{
					SendValid << "Mailbox for " << Parameters << " does not exist. Sorry.|";
					HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
				}
				LeaveCriticalSection(&cs);
			}
			else
			{
				SendValid << "You must specify a message.|";
				HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
			}
		}
		else
		{
			SendValid << "You must specify a Nick to send to!|";
			HexHub::Data::SendTo(userId, SendValid.str().c_str(), SendValid.str().length());
		}
	}
}

const char * HEXHUBAPI GetAboutText(void) {
	return "Mikhail Polenin, 2007 - 2008, Release date: " __DATE__ " " __TIME__ "";
}

void HEXHUBAPI UnloadPlugin()
{
	delete Msgbox;
	DeleteCriticalSection(&cs);
}

void HEXHUBAPI onChkCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char usernick[40];
	char botnick[40];
	HexHub::Users::ClientNickFromId(userId, usernick);
	EnterCriticalSection(&cs);
	if(Msgbox->Find(usernick))
	{
		if(HexHub::Users::NickFromId(0, botnick))
		{
			std::stringstream Sendstr;
			Sendstr << "<Hermes> " << Msgbox->GetMessages(usernick) << "|";
			HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
		}
	}
	else
	{
		if(HexHub::Users::NickFromId(0, botnick))
		{
			std::stringstream str;
			str << "< " << botnick << "> You do not have a mailbox.|";
			HexHub::Data::SendTo(userId, str.str().c_str(), str.str().length());
		}
	}
	LeaveCriticalSection(&cs);
}

void HEXHUBAPI onDelCommand(DWORD dwCommandId, DWORD userId, LPVOID rights, const char *commandParameters, BOOL isPM)
{
	char usernick[40];
	char botnick[40];
	HexHub::Users::ClientNickFromId(userId, usernick);
	EnterCriticalSection(&cs);
	if(Msgbox->Find(usernick))
	{
		if(HexHub::Users::NickFromId(0, botnick))
		{
			std::stringstream Sendstr;
			if(Msgbox->DelMessages(usernick))
			{
				Sendstr << "<Hermes> Successfully Cleared Your Mailbox!|";
				HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
			}
			else
			{
				Sendstr << "<Hermes> Error Clearing Mailbox!|";
				HexHub::Data::SendTo(userId, Sendstr.str().c_str(), Sendstr.str().length());
			}
		}
	}
	else
	{
		if(HexHub::Users::NickFromId(0, botnick))
		{
			std::stringstream str;
			str << "< " << botnick << "> You do not have a mailbox.|";
			HexHub::Data::SendTo(userId, str.str().c_str(), str.str().length());
		}
	}
	LeaveCriticalSection(&cs);
}